def plot_image(i, predictions_array, true_label, img): 
	predictions_array, true_label, img = predictions_array, true_label[i], img[i]
	plt.grid(False)
	plt.xticks([])
	plt.yticks([])
	
	plt.imshow(img, cmap=plt.cm.binary)
	
	predicted_label = np.argmax(predictions_array) 
	if predicted_label == true_label: 
		color = 'blue'
	else: 
		color = 'red'
		
	plt.xlabel("{} {:2.0f}% ({})".format(class_names[predicted_label], 
								100*np.max(predictions_array), 
								class_names[true_label]),
								color=color) 
		
	
def plot_value_array(i, predictions_array, true_label): 
	predictions_array, true_label = predictions_array, true_label[i]
	plt.grid(False) 
	plt.xticks(range(10))
	plt.yticks([])
	thisplot = plt.bar(range(10), predictions_array, color="#777777") 
	plt.ylim([0, 1]) 
	predicted_label = np.argmax(predictions_array) 
	
	thisplot[predicted_label].set_color('red') 
	thisplot[true_label].set_color('blue') 		

	

# tensorflow libraries 
import tensorflow as tf
from tensorflow import keras
from tensorflow.keras import datasets, layers, models
# helper libraries 
import numpy as np
import matplotlib.pyplot as plt
import cv2

## return version number 
print(tf.__version__)

## import fashion mnist database 
fashion_mnist = keras.datasets.fashion_mnist
(train_images, train_labels), (test_images, test_labels) = fashion_mnist.load_data()
saveable_images = test_images


print(train_images)
print(train_labels)

## labels are stored as numbers so we shall create names for them for visualisation 
class_names = ['T-shirt/top', 'Trouser', 'Pullover', 'Dress', 'Coat', 'Sandal', 'Shirt', 'Sneaker', 'Bag', 'Ankle boot']

## report the number and size of the training images 
print(train_images.shape)

len(train_labels)

train_labels 

test_images.shape

len(test_labels)

#### Data processing ####

## display the first training image 
plt.figure()
plt.imshow(train_images[0]) 
plt.colorbar()
plt.grid(False) 
plt.show()

## display the first 25 training images 
plt.figure(figsize=(10, 10)) 
for i in range(25): 
	plt.subplot(5,5,i+1)
	plt.xticks([])
	plt.yticks([])
	plt.grid(False)
	plt.imshow(train_images[i], cmap=plt.cm.binary)
	plt.xlabel(class_names[train_labels[i]])
	
plt.show()

## build a model of a neural network for training 
## layer one reformats the image into a flat stack of pixels 

# the second call 'dense' creates a layer of 128 densely or fully connected neural layers 
# the last call 'desnse' returns a logits array of length 10, each node contains a score that indicates the current image belonging to one of the 10 classes 

printable_test_images = test_images
train_images, test_images = train_images[..., np.newaxis]/255.0, test_images[..., np.newaxis]/255.0

# Create a CNN base 
model = models.Sequential()
model.add(layers.Conv2D(32, [3,3], activation='relu', input_shape=(28,28,1)))
model.add(layers.Conv2D(64, [3,3], activation='relu'))
model.add(layers.MaxPooling2D(pool_size=(2,2)))
model.add(layers.Dropout(0.25))
model.add(layers.Flatten())
model.add(layers.Dense(128, activation='relu'))
model.add(layers.Dropout(0.5))
model.add(layers.Dense(10))

#model = keras.Sequential([#
#	keras.layers.Flatten(input_shape=(28, 28)),
#	keras.layers.Dense(128, activation='relu'), 
#	keras.layers.Dense(10)
#])

## compile the model 
model.compile(optimizer='adam', 
				loss=tf.keras.losses.SparseCategoricalCrossentropy(from_logits=True), 
				metrics=['accuracy'])
				
## Train the model 

# 1. feed the training data to the model 
# 2. model learns to associate images and labels
# 3. ask the model to make predictions about a test set 
# 4. verify the predictions using the laebls from the test_labels set 

# Feed the model 

# start training by calling model.fit 
model.fit(train_images, train_labels, epochs=10) 

# Evaluate accuracy
test_loss, test_acc = model.evaluate(test_images, test_labels, verbose=2) 
print('\nTest accuracy:', test_acc) 

## Make predictions 
# use a softmax layer to convert the logits to probabilities that are easier to interpret 

probability_model = tf.keras.Sequential([model, tf.keras.layers.Softmax()]) 
predictions = probability_model.predict(test_images)

# report the first prediction 
predictions[0] 

# extract the class with the highest probability 
np.argmax(predictions[0])

# report the label for the first image in the test set so we can compare 
test_labels[0] 

## Create a graph to examine the full set of 10 class predictions
	
## Verify predictions 
# show the first X images 

num_rows = 5
num_cols = 3 
num_images = num_rows*num_cols
plt.figure(figsize=(2*2*num_cols, 2*num_rows))
for i in range(num_images): 
	image_name = "cloth_classifier/img_" + str(i) + ".png"
	plt.subplot(num_rows, 2*num_cols, 2*i+1) 
	plot_image(i, predictions[i], test_labels, printable_test_images)
	print("image array: ") 
	print(saveable_images[i])
	print("###")
	cv2.imwrite(image_name,saveable_images[i])
	plt.subplot(num_rows, 2*num_cols, 2*i+2) 
	plot_value_array(i, predictions[i], test_labels) 
plt.tight_layout()
plt.show() 

## save the model 
model.save('models/cloth') 


## convert Keras model tensorflow light format 
converter = tf.lite.TFLiteConverter.from_keras_model(model)
tflite_model = converter.convert()

# save the TF light model 
with tf.io.gfile.GFile('cloth.tflite', 'wb') as f: 
	f.write(tflite_model)
	
## convert Keras model tensorflow light format 
converter = tf.lite.TFLiteConverter.from_keras_model(probability_model)
tflite_model = converter.convert()

# save the TF light model 
with tf.io.gfile.GFile('probability_cloth.tflite', 'wb') as f: 
	f.write(tflite_model)